#!/usr/bin/env ruby
#
# Relays public messages from an IRC channel to a Campfire room.
# Adjust the configuration at the bottom of this file, then run
# the file to begin relaying.
#
# (c) 2008 Sam Stephenson <sam at 37signals dot com>
# Unsupported and freely distributable under the terms of an
# MIT-style license.

$:.unshift File.join(File.dirname(__FILE__), "ext")
require "irc/lib/irc"
require "marshmallow"

module Campfire
  class RelayHandler < Event::Handler
    def on_001
      request(:join, options[:channel])
    end
  
    def on_privmsg
      campfire.say(response.text) if channel
    end
    
    def on_ctcp_action
      campfire.say("*#{memo[:text]}*") and stop if channel
    end
  end

  class RelayBot < IRC::Client
    handler :relay
    
    def initialize(*)
      super
      @campfire = Marshmallow.new(options[:campfire])
      @campfire.login(options[:campfire].merge(:method => :login))
    end

    attr_reader :campfire
  end
end

if __FILE__ == $0
  require "yaml"
  Campfire::RelayBot.new(YAML.load(DATA)).start
end

__END__

:server: irc.example.com
:port: 6667
:channel: "#example"

:campfire:
  :domain: your-campfire-subdomain
  :ssl: true
  :username: email@address.com
  :password: password
  :room: 12345
