module IRC
  class Client
    attr_reader :options, :server, :port

    def initialize(options = { })
      @options = options
      initialize_handlers
    end
    
    def start
      connect(options[:server], options[:port]) do
        connection.each_response do |response|
          handle(response)
        end
      end
    end
    
    def stop
      connection.disconnect
    end
    
    def method_missing(method, *args, &block)
      handlers.each do |handler|
        if handler.class.public_method_defined?(method)
          return handler.send(method, *args, &block)
        end
      end
      super
    end
    
    protected
      attr_reader :connection, :response
    
      def connect(server, port)
        unless connection
          @server, @port, @connection = server, port, Connection.new(server, port)
          fire(:connection_established)
          yield
        end
      ensure
        if connection
          connection.disconnect
          @connection = nil
          fire(:disconnected)
        end
      end
    
      def handle(response)
        @response = response
        fire(response.name.downcase)
      ensure
        @response = nil
      end
    
      def request(command, *args)
        connection.request(command, *args)
      end
      
      def normalize(irc_string)
        irc_string.downcase # FIXME: only works when CASEMAPPING=ascii
      end
  end
end
