require File.join(File.dirname(__FILE__), "test_helper")

module EventTest
  class Broker
    include Event::Broker
    handler :foo, :bar
    
    def initialize
      initialize_handlers
    end
  end

  class TestHandler < Event::Handler
    class << self
      attr_reader :log
    
      def reset
        @log = []
      end
    end
  
    def log(*message)
      TestHandler.log << [self.class, *message]
    end
  end

  class FooHandler < TestHandler
    def on_one
      log :one
    end
  
    def on_two
      log :two
    end
  
    def on_three
      log :three
      stop
    end
  end

  class BarHandler < TestHandler
    def on_two
      log :two
    end
  
    def on_three
      log :three
    end
  
    def on_four
      log :four, memo[:message]
    end
  end

  class EventTest < Test::Unit::TestCase
    def setup
      TestHandler.reset
      @broker = Broker.new
    end
  
    def test_fire_with_one_handler
      assert @broker.fire(:one)
      assert_log [FooHandler, :one]
    end
  
    def test_fire_with_two_handlers
      assert @broker.fire(:two)
      assert_log [FooHandler, :two], [BarHandler, :two]
    end
  
    def test_fire_with_handler_that_calls_stop
      assert !@broker.fire(:three)
      assert_log [FooHandler, :three]
    end
  
    def test_fire_with_memo
      assert @broker.fire(:four, :message => "hello")
      assert_log [BarHandler, :four, "hello"]
    end
  
    protected
      def assert_log(*messages)
        assert_equal messages, TestHandler.log
      end
  end
end
