require File.join(File.dirname(__FILE__), "test_helper")

module InheritableAccessorsTest
  class Base
    include InheritableAccessors

    inheritable_array_accessor :handler
    handler :zero

    inheritable_hash_accessor :option
    option :zero => 0
  end

  class A < Base
    handler :one, :two, :three
    option :one => 1, :two => 2, :three => 3
  end

  class B < Base
    handler :four
    option :four => 4, :zero => -1
  end

  class C < A
    handler :five
    option :five => 5
  end

  class Base2
    include InheritableAccessors
    inheritable_array_accessor :handler
    inheritable_hash_accessor :option
  end

  class D < Base2
    handler :six
    option :six => 6
  end

  class InheritableAccessorsTest < Test::Unit::TestCase
    def test_inheritable_array_accessor
      assert_equal [:zero], Base.handlers
      assert_equal [:zero, :one, :two, :three], A.handlers
      assert_equal [:zero, :four], B.handlers
      assert_equal [:zero, :one, :two, :three, :five], C.handlers
      assert_equal [], Base2.handlers
      assert_equal [:six], D.handlers
    end
  
    def test_inheritable_hash_accessor
      assert_equal({ :zero => 0 }, Base.options)
      assert_equal({ :zero => 0, :one => 1, :two => 2, :three => 3 }, A.options)
      assert_equal({ :zero => -1, :four => 4 }, B.options)
      assert_equal({ :zero => 0, :one => 1, :two => 2, :three => 3, :five => 5 }, C.options)
      assert_equal({ }, Base2.options)
      assert_equal({ :six => 6 }, D.options)
    end
  end
end
